import 'package:w3c_event_source/event_source.dart';
import 'dart:async';
import 'package:http/http.dart' as http;
import 'package:meta/meta.dart';

class Mercure {
  final String hub_url;
  final String token;

  Mercure({@required this.hub_url, this.token, http.Client client})
      : assert(hub_url != null),
        assert(token != null); 
  

  /// Subscribe to one mercure topic
  StreamSubscription<MessageEvent> subscribeTopic(
      {@required String topic,
      @required void onData(MessageEvent event),
      Function onError,
      void onDone(),
      bool cancelOnError}) {
    subscribeTopics(
        topics: <String>[topic],
        onData: onData,
        onError: onError,
        onDone: onDone,
        cancelOnError: cancelOnError);
  }

  /// Subscribe to a list of mercure topics
  StreamSubscription<MessageEvent> subscribeTopics(
      {@required List<String> topics,
      @required void onData(MessageEvent event),
      Function onError,
      void onDone(),
      bool cancelOnError}) {
    String params = topics.map((topic) => 'topic=$topic&').join();
    EventSource eventSource = EventSource(Uri.parse('$hub_url?$params'));
        
      eventSource.events.listen(onData,
          onError: onError, onDone: onDone, cancelOnError: cancelOnError);
  }

  /// Publish data in mercure topic
  Future<int> publish({@required String topic, @required String data}) async {
    var response = await http.post(this.hub_url,
        body: {'topic': topic, 'data': data},
        headers: {'Authorization': 'Bearer $token'});
    return response.statusCode;
  }
}
